<?php

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\IOFactory;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Writer\Xls;

class WooCommerce_Ultimate_Pricing_Exporter extends WooCommerce_Ultimate_Pricing 
{
    protected $plugin_name;
    protected $version;
    protected $options;

    /**
     * Construct Store Locator Admin Class
     * @author Daniel Barenkamp
     * @version 1.0.0
     * @since   1.0.0
     * @link    http://www.welaunch.io
     * @param   string                         $plugin_name
     * @param   string                         $version    
     */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->notice = "";
    }

    public function init()
    {
        global $woocommerce_ultimate_pricing_options;
        $this->options = $woocommerce_ultimate_pricing_options;

        add_action( 'admin_notices', array($this, 'notice' ));

        if ( ! is_admin() || !is_user_logged_in() || !current_user_can('manage_options')) {
            $this->notice .= esc_html__("You are not an admin", 'woocommerce-ultimate-pricing');
            return FALSE;
        }

        global $wp_roles;
        if(isset($wp_roles->roles)) {
            $this->userRoles = array_keys($wp_roles->roles);
        }

        $products = $this->get_products();
        if(empty($products)) {
            $this->notice .= esc_html__("No products to Export", 'woocommerce-ultimate-pricing');
            return FALSE;
        }
        
        if($this->build_export($products)) {
            $this->notice .= esc_html__("Your Store Export is ready. The Download should start automatically.", 'woocommerce-ultimate-pricing');
        } else {
            $this->notice .= esc_html__("Something was wrong with the export generation ...", 'woocommerce-ultimate-pricing');
        };
        
    }

    public function get_products()
    {
        $args = array(
            'posts_per_page'   => -1,
            'post_type'        => array('product', 'product_variation'),
            'post_status'      => 'publish',
        );
        $posts = get_posts( $args );

        $products = array();
        foreach ($posts as $post) {

            $id = $post->ID;
            // price meta key is sale price when exists otherwise it is regular
            $postMetas = get_post_meta($post->ID);

            $customUserRolePrices = array();
            if(isset($postMetas['woocommerce_ultimate_pricing_prices']) && !empty($postMetas['woocommerce_ultimate_pricing_prices'][0])) {
                $customUserRolePrices = unserialize($postMetas['woocommerce_ultimate_pricing_prices'][0]);
            }

            $products[$id]['id'] = $id;
            $products[$id]['type'] = $post->post_type;
            $products[$id]['name'] = $post->post_title;
            $products[$id]['sku'] = isset($postMetas['_sku']) && !empty($postMetas['_sku'][0]) ? $postMetas['_sku'][0] : '';
            $products[$id]['regular_price'] = isset($postMetas['_regular_price']) && !empty($postMetas['_regular_price'][0]) ? $postMetas['_regular_price'][0] : '';
            $products[$id]['sale_price'] = isset($postMetas['_sale_price']) && !empty($postMetas['_sale_price'][0]) ? $postMetas['_sale_price'][0] : '';


            foreach ($this->userRoles as $userRole) {
                if(isset($customUserRolePrices[$userRole]) && !empty($customUserRolePrices[$userRole])) {
                    $products[$id][$userRole . '_price'] = $customUserRolePrices[$userRole];
                } else {
                    $products[$id][$userRole . '_price'] = '';
                }
            }
        }

        return $products;
    }

    public function build_export($products)
    {
        $excelExt = '.xlsx';
        $writer = 'Xlsx';

        $useExcel2007 = $this->get_option('excel2007');

        if($useExcel2007 == "1") {
            $excelExt = '.xls';
            $writer = 'Xls';
        }

        $spreadsheet = new Spreadsheet();

        // Set document properties
        $spreadsheet->getProperties()->setCreator("weLaunch")
                                     ->setLastModifiedBy("weLaunch")
                                     ->setTitle("Prices Export (".date('Y.m.d - H:i:s').")")
                                     ->setSubject("Prices export")
                                     ->setDescription("Prices export.")
                                     ->setKeywords("woocommerce prices");
        // Add some data
        // A note from the manual: In PHPExcel column index is 0-based while row index is 1-based. That means 'A1' ~ (0,1)
        $row = 1; // 1-based index
        $firstLine = true;
        foreach ($products as $fields) {
            $col = 1;
            if ($firstLine) {
                $keys = array_keys($fields);
                foreach ($keys as $key) {
                    $spreadsheet->getActiveSheet()->setCellValueByColumnAndRow($col, $row, $key);
                    $col++;
                }
                $row++;
                $col = 1;
                $firstLine = false;
            } 
           
            foreach($fields as $key => $value) {

                $spreadsheet->getActiveSheet()->setCellValueByColumnAndRow($col, $row, $value);
                $col++;
            }
            $row++;
        }

        // Rename worksheet
        $spreadsheet->getActiveSheet()->setTitle('Export ('.date('Y.m.d - H.i.s').')');
        // Set active sheet index to the first sheet, so Excel opens this as the first sheet
        $spreadsheet->setActiveSheetIndex(0);
        // Redirect output to a client’s web browser (Excel2007) using ob_end_clean();
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment;filename="prices-export_' . date('Y-m-d_H-i-s') . $excelExt . '"');
        header('Cache-Control: max-age=0');
        // If you're serving to IE 9, then the following may be needed
        header('Cache-Control: max-age=1');
        // If you're serving to IE over SSL, then the following may be needed
        header ('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); // Date in the past
        header ('Last-Modified: '.gmdate('D, d M Y H:i:s').' GMT'); // always modified
        header ('Cache-Control: cache, must-revalidate'); // HTTP/1.1
        header ('Pragma: public'); // HTTP/1.0

        $writer = IOFactory::createWriter($spreadsheet, $writer);
        $writer->save('php://output');

        exit();
        return TRUE;
    }

    public function notice()
    {
        ?>
        <div class="notice notice-success is-dismissible">
            <p><?php echo $this->notice ?></p>
        </div>
        <?php
    }
}